//
//  LBColoredHorizontalTabView.m
//  LeatherBound
//
//  Created by Michael Potter on 9/1/11.
//  Copyright (c) 2011 Michael Potter. All rights reserved.
//

#import "LBColoredHorizontalTabView.h"

#pragma mark Internal Definitions

static NSMutableDictionary *sCachedTabImages = nil;

#pragma mark - Class Extension

@interface LBColoredHorizontalTabView ()

@property (nonatomic, strong) UIImageView *tabImageView;
@property (nonatomic, copy, readonly) NSString *typeString;

- (BOOL)prerenderedTabImageExistsForColor:(UIColor *)color;
- (NSString *)prerenderedTabImageKeyForColor:(UIColor *)color;
- (void)renderTabImageForColor:(UIColor *)color;

@end

@implementation LBColoredHorizontalTabView

@synthesize tabColor;
@synthesize type;
@synthesize tabImageView;

#pragma mark - Property Accessors

- (void)setTabColor:(UIColor *)newTabColor
{
	tabColor = newTabColor;

	if (! [self prerenderedTabImageExistsForColor:self.tabColor])
	{
		[self renderTabImageForColor:self.tabColor];
	}

	self.tabImageView.image = [sCachedTabImages objectForKey:[self prerenderedTabImageKeyForColor:self.tabColor]];
}

- (void)setType:(LBColoredHorizontalTabViewType)newType
{
	type = newType;

	if (type == LBColoredHorizontalTabViewTypeArrow)
	{
		self.tabImageView.contentMode = UIViewContentModeRight;
	}
}

- (NSString *)typeString
{
	NSString *stringForType = nil;

	if (self.type == LBColoredHorizontalTabViewTypeNormal)
	{
		stringForType = @"Normal";
	}
	else if (self.type == LBColoredHorizontalTabViewTypeArrow)
	{
		stringForType = @"Arrow";
	}

	return stringForType;
}

#pragma mark - LBColoredHorizontalTabView Methods (Private)

- (BOOL)prerenderedTabImageExistsForColor:(UIColor *)color
{
	UIImage *prerenderedTabImage = [sCachedTabImages objectForKey:[self prerenderedTabImageKeyForColor:color]];

	return (prerenderedTabImage != nil);
}

- (NSString *)prerenderedTabImageKeyForColor:(UIColor *)color
{
	NSString *tabImageKey = nil;

	CGFloat colorHue = 0.0f;
	CGFloat colorSaturation = 0.0f;
	CGFloat colorBrightness = 0.0f;
	CGFloat colorAlpha = 0.0f;

	BOOL didRetrieveColorComponents = [color getHue:&colorHue saturation:&colorSaturation brightness:&colorBrightness alpha:&colorAlpha];

	if (didRetrieveColorComponents)
	{
		tabImageKey = [NSString stringWithFormat:@"%@_%f_%f_%f_%f", self.typeString, colorHue, colorSaturation, colorBrightness, colorAlpha];
	}

	return tabImageKey;
}

- (void)renderTabImageForColor:(UIColor *)color
{
	CGRect rect = self.bounds;

	UIGraphicsBeginImageContextWithOptions(rect.size, NO, 0.0f);

	CGContextRef context = UIGraphicsGetCurrentContext();

	CGContextTranslateCTM(context, 0.0f, rect.size.height);
	CGContextScaleCTM(context, 1.0f, -1.0f);

	CGContextSaveGState(context);

	NSString *imageBaseName = [NSString stringWithFormat:@"%@%@", NSStringFromClass([self class]), self.typeString];
	NSString *maskImageName = [NSString stringWithFormat:@"%@%@", imageBaseName, @"Mask"];

	UIImage *maskImage = [UIImage imageNamed:maskImageName];
	CGContextClipToMask(context, rect, maskImage.CGImage);

	CGContextSetFillColorWithColor(context, self.tabColor.CGColor);
	CGContextFillRect(context, rect);

	CGContextRestoreGState(context);

	CGContextSetBlendMode(context, kCGBlendModeHardLight);

	NSString *baseImageName = [NSString stringWithFormat:@"%@%@", imageBaseName, @"Base"];

	UIImage *baseImage = [UIImage imageNamed:baseImageName];
	CGContextDrawImage(context, rect, baseImage.CGImage);

	UIImage *renderedImage = UIGraphicsGetImageFromCurrentImageContext();

	UIGraphicsEndImageContext();

	[sCachedTabImages setValue:renderedImage forKey:[self prerenderedTabImageKeyForColor:color]];
}

#pragma mark - NSObject Methods

+ (void)initialize
{
	if (self == [LBColoredHorizontalTabView class])
	{
		sCachedTabImages = [NSMutableDictionary dictionary];
	}
}

#pragma mark - Protocol Implementations

#pragma mark - NSCoding Methods

- (id)initWithCoder:(NSCoder *)decoder
{
	self = [super initWithCoder:decoder];

	if (self != nil)
	{
		self.type = LBColoredHorizontalTabViewTypeNormal;

		self.tabImageView = [[UIImageView alloc] initWithFrame:self.frame];
		self.tabImageView.autoresizingMask = UIViewAutoresizingFlexibleWidth;

		[self addSubview:self.tabImageView];
	}

	return self;
}

@end